package at.xaxa.ledger.ui.edit

import android.util.Log
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.DropdownMenuItem
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.ExposedDropdownMenuBox
import androidx.compose.material3.ExposedDropdownMenuDefaults.TrailingIcon
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableIntStateOf
import androidx.compose.runtime.mutableLongStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.lifecycle.viewmodel.compose.viewModel
import at.xaxa.ledger.ui.AppViewModelProvider
import at.xaxa.ledger.ui.ButtonDanger
import at.xaxa.ledger.ui.ButtonSuccess
import at.xaxa.ledger.ui.DatePickerDocked

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun Edit(modifier: Modifier = Modifier, onCardClick: () -> Unit, editViewModel : EditViewModel = viewModel(factory = AppViewModelProvider.Factory)) {
	val entry = editViewModel.editUiState.entry
    val categories by editViewModel.categoryUiState.categories.collectAsState(initial = emptyList())

    Log.w("xaver", entry.name)
    Log.w("xaver", entry.amount.toString())
    Log.w("xaver", entry.date.toString())
    Log.w("xaver", editViewModel.categoryUi.categories.categoryName)
    Log.w("xaver", entry.categoryID.toString())


    var name by remember { mutableStateOf(entry.name) }
    var spending by remember { mutableStateOf(entry.amount.toString()) }
    var selectedDate by remember { mutableLongStateOf(entry.date) }
    var expanded by remember { mutableStateOf(false) }
    var selectedItem by remember { mutableStateOf(editViewModel.categoryUi.categories.categoryName) }
    var selectedCategory by remember { mutableIntStateOf(entry.categoryID) }

    Column(
        modifier = modifier.fillMaxSize()
            .padding(16.dp, 0.dp),
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        Column(
            Modifier.weight(1f)
        ) {
            OutlinedTextField(
                value = name,
                onValueChange = { name = it },
                label = { Text("Name") },
                modifier = Modifier
                    .fillMaxWidth()
            )
            OutlinedTextField(
                value = spending,
                onValueChange = { spending = it },
                label = { Text("Spending") },
                modifier = Modifier
                    .fillMaxWidth()
            )

            ExposedDropdownMenuBox(
                expanded = expanded,
                onExpandedChange = { expanded = it }
            ) {
                // TextField to display the selected item and trigger the dropdown
                OutlinedTextField(
                    value = selectedItem,
                    onValueChange = {},
                    label = { Text("Category") },
                    readOnly = true,
                    trailingIcon = {
                        TrailingIcon(expanded = expanded)
                    },
                    modifier = Modifier
                        .menuAnchor()
                        .fillMaxWidth()
                )

                // Dropdown menu
                ExposedDropdownMenu(
                    expanded = expanded,
                    onDismissRequest = { expanded = false }
                ) {
                    categories.forEach { item ->
                        DropdownMenuItem(
                            text = { Text(text = item.categoryName) },
                            onClick = {
                                selectedItem = item.categoryName
                                selectedCategory = item._id
                                expanded = false
                            }
                        )
                    }
                }
            }

            DatePickerDocked{
                    dateMilis -> selectedDate = dateMilis
            }
        }

        Column(
            modifier = Modifier.fillMaxWidth(),
            horizontalAlignment = Alignment.CenterHorizontally // Center buttons horizontally
        ) {
            ButtonDanger(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(bottom = 8.dp),
                text = "Delete",
                onClick = { onCardClick() }
            )
            ButtonSuccess(
                modifier = Modifier.fillMaxWidth(), // Add spacing between buttons
                text = "Done",
                onClick = { onCardClick() }
            )
        }
    }
}
