package at.xaxa.ledger.ui.edit

import android.util.Log
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.setValue
import androidx.lifecycle.SavedStateHandle
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import at.xaxa.ledger.data.Entry
import at.xaxa.ledger.data.EntryRepository
import at.xaxa.ledger.data.db.Category.CategoryEntity
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.flowOf
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

data class CategoryListUIState(val categories: Flow<List<CategoryEntity>> = flowOf(emptyList()))
data class CategoryUIState(val categories: CategoryEntity = CategoryEntity(0,"",0))


data class EditUI(
    val entry: Entry = Entry(0, "", 0.0f, 0, 0)
)

class EditViewModel(private val savedStateHandle: SavedStateHandle,
                           private val entryRepository: EntryRepository) : ViewModel() {

    private val entryId: Int = checkNotNull(savedStateHandle["entryId"])


    var editUiState by mutableStateOf(EditUI())
    var categoryUiState by mutableStateOf(CategoryListUIState())
    var categoryUi by mutableStateOf(CategoryUIState())


    init {
        viewModelScope.launch {
            val entry = withContext(Dispatchers.IO) {
                entryRepository.findEntryById(entryId)
            }
            editUiState = EditUI(entry)

            findCategoryByID(entry.categoryID)
        }
    }

    fun updateEntry(entry: Entry) {
        editUiState = editUiState.copy(entry=entry)
    }
    fun onDeleteEntry(entry: Entry) {
        viewModelScope.launch {
            entryRepository.deleteEntry(entry)
        }
    }
    fun saveEntry() {
        viewModelScope.launch {
            entryRepository.updateEntry(editUiState.entry)
        }
    }
    fun getAllCategories() {
        viewModelScope.launch {
            val categories = withContext(Dispatchers.IO) {
                entryRepository.getAllCategories()
            }
            categoryUiState = CategoryListUIState(categories)

        }

    }
    fun findCategoryByID(categoryId: Int) {
        viewModelScope.launch {
            val category = withContext(Dispatchers.IO) {
                entryRepository.findCategoryById(categoryId)
            }
            categoryUi = CategoryUIState(category)

        }

    }

}